@@ -1,21 +1,21 @@
-/* Copyright (C) 1996-1998, 2000-2007, 2009, 2011-2015
- *   Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1996-1998,2000-2007,2009,2011-2015,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -34,33 +34,36 @@
 #include <string.h>
 #endif
 #include <unistd.h>
-#include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/types.h>
 #ifdef HAVE_UNIX_DOMAIN_SOCKETS
 #include <sys/un.h>
 #endif
 #include <netinet/in.h>
+#include <netinet/tcp.h>
 #include <netdb.h>
 #include <arpa/inet.h>
 
 #include <gmp.h>
 
-#include "libguile/_scm.h"
-#include "libguile/arrays.h"
-#include "libguile/feature.h"
-#include "libguile/fports.h"
-#include "libguile/strings.h"
-#include "libguile/vectors.h"
-#include "libguile/dynwind.h"
-#include "libguile/srfi-13.h"
-
-#include "libguile/validate.h"
-#include "libguile/socket.h"
-
-#if SCM_ENABLE_DEPRECATED == 1
-# include "libguile/deprecation.h"
-#endif
+#include "arrays.h"
+#include "async.h"
+#include "bytevectors.h"
+#include "dynwind.h"
+#include "feature.h"
+#include "fports.h"
+#include "gsubr.h"
+#include "list.h"
+#include "modules.h"
+#include "numbers.h"
+#include "pairs.h"
+#include "srfi-13.h"
+#include "strings.h"
+#include "symbols.h"
+#include "syscalls.h"
+#include "vectors.h"
 
+#include "socket.h"
 
 
 #if defined (HAVE_UNIX_DOMAIN_SOCKETS) && !defined (SUN_LEN)
@@ -160,7 +163,7 @@ SCM_DEFINE (scm_inet_makeaddr, "inet-makeaddr", 2, 0, 0,
   \
   for (i = 0; i < 8; i++)\
     {\
-      scm_t_uint8 c = (addr)[i];\
+      uint8_t c = (addr)[i];\
       \
       (addr)[i] = (addr)[15 - i];\
       (addr)[15 - i] = c;\
@@ -173,8 +176,8 @@ SCM_DEFINE (scm_inet_makeaddr, "inet-makeaddr", 2, 0, 0,
 #else
 #define FLIPCPY_NET_HOST_128(dest, src) \
 { \
-  const scm_t_uint8 *tmp_srcp = (src) + 15; \
-  scm_t_uint8 *tmp_destp = (dest); \
+  const uint8_t *tmp_srcp = (src) + 15; \
+  uint8_t *tmp_destp = (dest); \
   \
   do { \
     *tmp_destp++ = *tmp_srcp--; \
@@ -183,8 +186,8 @@ SCM_DEFINE (scm_inet_makeaddr, "inet-makeaddr", 2, 0, 0,
 #endif
 
 
-#if (SIZEOF_SCM_T_BITS * SCM_CHAR_BIT) > 128
-#error "Assumption that scm_t_bits <= 128 bits has been violated."
+#if (SIZEOF_UINTPTR_T * SCM_CHAR_BIT) > 128
+#error "Assumption that uintptr_t <= 128 bits has been violated."
 #endif
 
 #if (SIZEOF_UNSIGNED_LONG * SCM_CHAR_BIT) > 128
@@ -198,7 +201,7 @@ SCM_DEFINE (scm_inet_makeaddr, "inet-makeaddr", 2, 0, 0,
 /* convert a 128 bit IPv6 address in network order to a host ordered
    SCM integer.  */
 static SCM
-scm_from_ipv6 (const scm_t_uint8 *src)
+scm_from_ipv6 (const uint8_t *src)
 {
   SCM result = scm_i_mkbig ();
   mpz_import (SCM_I_BIG_MPZ (result),
@@ -214,7 +217,7 @@ scm_from_ipv6 (const scm_t_uint8 *src)
 /* convert a host ordered SCM integer to a 128 bit IPv6 address in
    network order.  */
 static void
-scm_to_ipv6 (scm_t_uint8 dst[16], SCM src)
+scm_to_ipv6 (uint8_t dst[16], SCM src)
 {
   if (SCM_I_INUMP (src))
     {
@@ -292,7 +295,7 @@ SCM_DEFINE (scm_inet_ntop, "inet-ntop", 2, 0, 0,
 		    );
   if (af == AF_INET)
     {
-      scm_t_uint32 addr4;
+      uint32_t addr4;
 
       addr4 = htonl (SCM_NUM2ULONG (2, address));
       result = inet_ntop (af, &addr4, dst, sizeof (dst));
@@ -302,7 +305,7 @@ SCM_DEFINE (scm_inet_ntop, "inet-ntop", 2, 0, 0,
     {
       char addr6[16];
 
-      scm_to_ipv6 ((scm_t_uint8 *) addr6, address);
+      scm_to_ipv6 ((uint8_t *) addr6, address);
       result = inet_ntop (af, &addr6, dst, sizeof (dst));
     }
 #endif
@@ -331,7 +334,7 @@ SCM_DEFINE (scm_inet_pton, "inet-pton", 2, 0, 0,
 {
   int af;
   char *src;
-  scm_t_uint32 dst[4];
+  uint32_t dst[4];
   int rv, eno;
 
   af = scm_to_int (family);
@@ -356,7 +359,7 @@ SCM_DEFINE (scm_inet_pton, "inet-pton", 2, 0, 0,
     return scm_from_ulong (ntohl (*dst));
 #ifdef HAVE_IPV6
   else if (af == AF_INET6)
-    return scm_from_ipv6 ((scm_t_uint8 *) dst);
+    return scm_from_ipv6 ((uint8_t *) dst);
 #endif
   else
     SCM_MISC_ERROR ("unsupported address family", family);
@@ -366,7 +369,12 @@ SCM_DEFINE (scm_inet_pton, "inet-pton", 2, 0, 0,
 
 SCM_SYMBOL (sym_socket, "socket");
 
-#define SCM_SOCK_FD_TO_PORT(fd) scm_fdes_to_port (fd, "r+0", sym_socket)
+static SCM
+scm_socket_fd_to_port (int fd)
+{
+  return scm_i_fdes_to_port (fd, scm_mode_bits ("r+0"), sym_socket,
+                             SCM_FPORT_OPTION_NOT_SEEKABLE);
+}
 
 SCM_DEFINE (scm_socket, "socket", 3, 0, 0,
             (SCM family, SCM style, SCM proto),
@@ -390,7 +398,7 @@ SCM_DEFINE (scm_socket, "socket", 3, 0, 0,
 	       scm_to_int (proto));
   if (fd == -1)
     SCM_SYSERROR;
-  return SCM_SOCK_FD_TO_PORT (fd);
+  return scm_socket_fd_to_port (fd);
 }
 #undef FUNC_NAME
 
@@ -412,7 +420,8 @@ SCM_DEFINE (scm_socketpair, "socketpair", 3, 0, 0,
   if (socketpair (fam, scm_to_int (style), scm_to_int (proto), fd) == -1)
     SCM_SYSERROR;
 
-  return scm_cons (SCM_SOCK_FD_TO_PORT (fd[0]), SCM_SOCK_FD_TO_PORT (fd[1]));
+  return scm_cons (scm_socket_fd_to_port (fd[0]),
+                   scm_socket_fd_to_port (fd[1]));
 }
 #undef FUNC_NAME
 #endif
@@ -708,6 +717,8 @@ SCM_DEFINE (scm_shutdown, "shutdown", 2, 0, 0,
    proc is the name of the original procedure.
    size returns the size of the structure allocated.  */
 
+#define MAX(A, B) ((A) >= (B) ? (A) : (B))
+
 static struct sockaddr *
 scm_fill_sockaddr (int fam, SCM address, SCM *args, int which_arg,
 		   const char *proc, size_t *size)
@@ -795,7 +806,7 @@ scm_fill_sockaddr (int fam, SCM address, SCM *args, int which_arg,
 	   connect/bind etc., to fail.  sun_path is always the last
 	   member of the structure.  */
 	addr_size = sizeof (struct sockaddr_un)
-	  + max (0, strlen (c_address) + 1 - (sizeof soka->sun_path));
+	  + MAX (0, strlen (c_address) + 1 - (sizeof soka->sun_path));
 	soka = (struct sockaddr_un *) scm_malloc (addr_size);
 	memset (soka, 0, addr_size);  /* for sun_len: see sin_len above. */
 	soka->sun_family = AF_UNIX;
@@ -833,7 +844,8 @@ SCM_DEFINE (scm_connect, "connect", 2, 1, 1,
 	    "Alternatively, the second argument can be a socket address object "
 	    "as returned by @code{make-socket-address}, in which case the "
 	    "no additional arguments should be passed.\n\n"
-	    "The return value is unspecified.")
+	    "Return true, unless the socket was configured to be non-blocking\n"
+            "and the operation has not finished yet.\n")
 #define FUNC_NAME s_scm_connect
 {
   int fd;
@@ -858,10 +870,12 @@ SCM_DEFINE (scm_connect, "connect", 2, 1, 1,
 
       free (soka);
       errno = save_errno;
+      if (errno == EINPROGRESS)
+        return SCM_BOOL_F;
       SCM_SYSERROR;
     }
   free (soka);
-  return SCM_UNSPECIFIED;
+  return SCM_BOOL_T;
 }
 #undef FUNC_NAME
 
@@ -1233,24 +1247,24 @@ SCM_DEFINE (scm_make_socket_address, "make-socket-address", 2, 0, 1,
 #undef FUNC_NAME
 
 
-SCM_DEFINE (scm_accept, "accept", 1, 0, 0, 
-            (SCM sock),
-	    "Accept a connection on a bound, listening socket.\n"
-	    "If there\n"
-	    "are no pending connections in the queue, wait until\n"
-	    "one is available unless the non-blocking option has been\n"
-	    "set on the socket.\n\n"
-	    "The return value is a\n"
-	    "pair in which the @emph{car} is a new socket port for the\n"
-	    "connection and\n"
-	    "the @emph{cdr} is an object with address information about the\n"
-	    "client which initiated the connection.\n\n"
+SCM_DEFINE (scm_accept4, "accept", 1, 1, 0, 
+            (SCM sock, SCM flags),
+	    "Accept a connection on a bound, listening socket.  If there\n"
+	    "are no pending connections in the queue, there are two\n"
+            "possibilities: if the socket has been configured as\n"
+            "non-blocking, return @code{#f} directly.  Otherwise wait\n"
+            "until a connection is available.  When a connection comes,\n"
+	    "the return value is a pair in which the @emph{car} is a new\n"
+            "socket port for the connection and the @emph{cdr} is an\n"
+            "object with address information about the client which\n"
+            "initiated the connection.\n\n"
 	    "@var{sock} does not become part of the\n"
 	    "connection and will continue to accept new requests.")
-#define FUNC_NAME s_scm_accept
+#define FUNC_NAME s_scm_accept4
 {
   int fd;
   int newfd;
+  int c_flags;
   SCM address;
   SCM newsock;
   socklen_t addr_size = MAX_ADDR_SIZE;
@@ -1258,18 +1272,30 @@ SCM_DEFINE (scm_accept, "accept", 1, 0, 0,
 
   sock = SCM_COERCE_OUTPORT (sock);
   SCM_VALIDATE_OPFPORT (1, sock);
+  c_flags = SCM_UNBNDP (flags) ? 0 : scm_to_int (flags);
+
   fd = SCM_FPORT_FDES (sock);
-  SCM_SYSCALL (newfd = accept (fd, (struct sockaddr *) &addr, &addr_size));
+  SCM_SYSCALL (newfd = accept4 (fd, (struct sockaddr *) &addr, &addr_size,
+                                c_flags));
   if (newfd == -1)
-    SCM_SYSERROR;
-  newsock = SCM_SOCK_FD_TO_PORT (newfd);
-  address = _scm_from_sockaddr (&addr, addr_size,
-				FUNC_NAME);
+    {
+      if (errno == EAGAIN || errno == EWOULDBLOCK)
+        return SCM_BOOL_F;
+      SCM_SYSERROR;
+    }
+  newsock = scm_socket_fd_to_port (newfd);
+  address = _scm_from_sockaddr (&addr, addr_size, FUNC_NAME);
 
   return scm_cons (newsock, address);
 }
 #undef FUNC_NAME
 
+SCM
+scm_accept (SCM sock)
+{
+  return scm_accept4 (sock, SCM_UNDEFINED);
+}
+
 SCM_DEFINE (scm_getsockname, "getsockname", 1, 0, 0, 
             (SCM sock),
 	    "Return the address of @var{sock}, in the same form as the\n"
@@ -1344,33 +1370,12 @@ SCM_DEFINE (scm_recv, "recv!", 2, 1, 0,
     flg = scm_to_int (flags);
   fd = SCM_FPORT_FDES (sock);
 
-#if SCM_ENABLE_DEPRECATED == 1
-  if (SCM_UNLIKELY (scm_is_string (buf)))
-    {
-      SCM msg;
-      char *dest;
-      size_t len;
+  SCM_VALIDATE_BYTEVECTOR (1, buf);
 
-      scm_c_issue_deprecation_warning
-	("Passing a string to `recv!' is deprecated, "
-	 "use a bytevector instead.");
-
-      len = scm_i_string_length (buf);
-      msg = scm_i_make_string (len, &dest, 0);
-      SCM_SYSCALL (rv = recv (fd, dest, len, flg));
-      scm_string_copy_x (buf, scm_from_int (0),
-			 msg, scm_from_int (0), scm_from_size_t (len));
-    }
-  else
-#endif
-    {
-      SCM_VALIDATE_BYTEVECTOR (1, buf);
-
-      SCM_SYSCALL (rv = recv (fd,
-			      SCM_BYTEVECTOR_CONTENTS (buf),
-			      SCM_BYTEVECTOR_LENGTH (buf),
-			      flg));
-    }
+  SCM_SYSCALL (rv = recv (fd,
+                          SCM_BYTEVECTOR_CONTENTS (buf),
+                          SCM_BYTEVECTOR_LENGTH (buf),
+                          flg));
 
   if (SCM_UNLIKELY (rv == -1))
     SCM_SYSERROR;
@@ -1410,35 +1415,12 @@ SCM_DEFINE (scm_send, "send", 2, 1, 0,
 
   fd = SCM_FPORT_FDES (sock);
 
-#if SCM_ENABLE_DEPRECATED == 1
-  if (SCM_UNLIKELY (scm_is_string (message)))
-    {
-      scm_c_issue_deprecation_warning
-	("Passing a string to `send' is deprecated, "
-	 "use a bytevector instead.");
+  SCM_VALIDATE_BYTEVECTOR (1, message);
 
-      /* If the string is wide, see if it can be coerced into a narrow
-	 string.  */
-      if (!scm_i_is_narrow_string (message)
-	  || !scm_i_try_narrow_string (message))
-	SCM_MISC_ERROR ("the message string is not 8-bit: ~s",
-                        scm_list_1 (message));
-
-      SCM_SYSCALL (rv = send (fd,
-			      scm_i_string_chars (message),
-			      scm_i_string_length (message),
-			      flg));
-    }
-  else
-#endif
-    {
-      SCM_VALIDATE_BYTEVECTOR (1, message);
-
-      SCM_SYSCALL (rv = send (fd,
-			      SCM_BYTEVECTOR_CONTENTS (message),
-			      SCM_BYTEVECTOR_LENGTH (message),
-			      flg));
-    }
+  SCM_SYSCALL (rv = send (fd,
+                          SCM_BYTEVECTOR_CONTENTS (message),
+                          SCM_BYTEVECTOR_LENGTH (message),
+                          flg));
 
   if (rv == -1)
     SCM_SYSERROR;
@@ -1496,52 +1478,28 @@ SCM_DEFINE (scm_recvfrom, "recvfrom!", 2, 3, 0,
 
   ((struct sockaddr *) &addr)->sa_family = AF_UNSPEC;
 
-#if SCM_ENABLE_DEPRECATED == 1
-  if (SCM_UNLIKELY (scm_is_string (buf)))
-    {
-      char *cbuf;
+  SCM_VALIDATE_BYTEVECTOR (1, buf);
 
-      scm_c_issue_deprecation_warning
-	("Passing a string to `recvfrom!' is deprecated, "
-	 "use a bytevector instead.");
+  if (SCM_UNBNDP (start))
+    offset = 0;
+  else
+    offset = scm_to_size_t (start);
 
-      scm_i_get_substring_spec (scm_i_string_length (buf),
-				start, &offset, end, &cend);
-
-      buf = scm_i_string_start_writing (buf);
-      cbuf = scm_i_string_writable_chars (buf);
-
-      SCM_SYSCALL (rv = recvfrom (fd, cbuf + offset,
-				  cend - offset, flg,
-				  (struct sockaddr *) &addr, &addr_size));
-      scm_i_string_stop_writing ();
-    }
+  if (SCM_UNBNDP (end))
+    cend = SCM_BYTEVECTOR_LENGTH (buf);
   else
-#endif
     {
-      SCM_VALIDATE_BYTEVECTOR (1, buf);
-
-      if (SCM_UNBNDP (start))
-	offset = 0;
-      else
-	offset = scm_to_size_t (start);
-
-      if (SCM_UNBNDP (end))
-	cend = SCM_BYTEVECTOR_LENGTH (buf);
-      else
-	{
-	  cend = scm_to_size_t (end);
-	  if (SCM_UNLIKELY (cend >= SCM_BYTEVECTOR_LENGTH (buf)
-			    || cend < offset))
-	    scm_out_of_range (FUNC_NAME, end);
-	}
-
-      SCM_SYSCALL (rv = recvfrom (fd,
-				  SCM_BYTEVECTOR_CONTENTS (buf) + offset,
-				  cend - offset, flg,
-				  (struct sockaddr *) &addr, &addr_size));
+      cend = scm_to_size_t (end);
+      if (SCM_UNLIKELY (cend >= SCM_BYTEVECTOR_LENGTH (buf)
+                        || cend < offset))
+        scm_out_of_range (FUNC_NAME, end);
     }
 
+  SCM_SYSCALL (rv = recvfrom (fd,
+                              SCM_BYTEVECTOR_CONTENTS (buf) + offset,
+                              cend - offset, flg,
+                              (struct sockaddr *) &addr, &addr_size));
+
   if (rv == -1)
     SCM_SYSERROR;
 
@@ -1611,35 +1569,12 @@ SCM_DEFINE (scm_sendto, "sendto", 3, 1, 1,
       flg = SCM_NUM2ULONG (5, SCM_CAR (args_and_flags));
     }
 
-#if SCM_ENABLE_DEPRECATED == 1
-  if (SCM_UNLIKELY (scm_is_string (message)))
-    {
-      scm_c_issue_deprecation_warning
-	("Passing a string to `sendto' is deprecated, "
-	 "use a bytevector instead.");
+  SCM_VALIDATE_BYTEVECTOR (1, message);
 
-      /* If the string is wide, see if it can be coerced into a narrow
-	 string.  */
-      if (!scm_i_is_narrow_string (message)
-	  || !scm_i_try_narrow_string (message))
-	SCM_MISC_ERROR ("the message string is not 8-bit: ~s",
-                        scm_list_1 (message));
-
-      SCM_SYSCALL (rv = sendto (fd,
-				scm_i_string_chars (message),
-				scm_i_string_length (message),
-				flg, soka, size));
-    }
-  else
-#endif
-    {
-      SCM_VALIDATE_BYTEVECTOR (1, message);
-
-      SCM_SYSCALL (rv = sendto (fd,
-				SCM_BYTEVECTOR_CONTENTS (message),
-				SCM_BYTEVECTOR_LENGTH (message),
-				flg, soka, size));
-    }
+  SCM_SYSCALL (rv = sendto (fd,
+                            SCM_BYTEVECTOR_CONTENTS (message),
+                            SCM_BYTEVECTOR_LENGTH (message),
+                            flg, soka, size));
 
   if (rv == -1)
     {
@@ -1722,6 +1657,14 @@ scm_init_socket ()
   scm_c_define ("SOCK_RDM", scm_from_int (SOCK_RDM));
 #endif
 
+  /* accept4 flags.  */
+#ifdef SOCK_CLOEXEC
+  scm_c_define ("SOCK_CLOEXEC", scm_from_int (SOCK_CLOEXEC));
+#endif
+#ifdef SOCK_NONBLOCK
+  scm_c_define ("SOCK_NONBLOCK", scm_from_int (SOCK_NONBLOCK));
+#endif
+
   /* setsockopt level.
 
      SOL_IP, SOL_TCP and SOL_UDP are defined on gnu/linux, but not on for
@@ -1804,6 +1747,14 @@ scm_init_socket ()
   scm_c_define ("MSG_DONTROUTE", scm_from_int (MSG_DONTROUTE));
 #endif
 
+  /* TCP options.  */
+#ifdef TCP_NODELAY
+  scm_c_define ("TCP_NODELAY", scm_from_int (TCP_NODELAY));
+#endif
+#ifdef TCP_CORK
+  scm_c_define ("TCP_CORK", scm_from_int (TCP_CORK));
+#endif
+
 #ifdef IP_ADD_MEMBERSHIP
   scm_c_define ("IP_ADD_MEMBERSHIP", scm_from_int (IP_ADD_MEMBERSHIP));
   scm_c_define ("IP_DROP_MEMBERSHIP", scm_from_int (IP_DROP_MEMBERSHIP));
@@ -1819,12 +1770,6 @@ scm_init_socket ()
 
   scm_add_feature ("socket");
 
-#include "libguile/socket.x"
+#include "socket.x"
 }
 
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
